# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-30 17:02
from django.db import migrations, models
import django.db.models.deletion
import django_attachments.models
import easy_thumbnails.fields


class Migration(migrations.Migration):

	initial = True

	dependencies = [
	]

	operations = [
		migrations.CreateModel(
			name='Attachment',
			fields=[
				('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
				('created', models.DateTimeField(db_index=True, editable=False, verbose_name='Created')),
				('updated', models.DateTimeField(db_index=True, editable=False, verbose_name='Updated')),
				('rank', models.IntegerField(verbose_name='Rank')),
				('original_name', models.CharField(max_length=255, verbose_name='Original name')),
				('file', easy_thumbnails.fields.ThumbnailerField(upload_to=django_attachments.models.upload_path_handler, verbose_name='File')),
				('filesize', models.BigIntegerField(verbose_name='File size')),
				('mimetype', models.CharField(blank=True, max_length=200, verbose_name='File type')),
				('image_width', models.IntegerField(blank=True, null=True, verbose_name='Image width')),
				('image_height', models.IntegerField(blank=True, null=True, verbose_name='Image height')),
			],
			options={
				'verbose_name_plural': 'Attachments',
				'verbose_name': 'Attachment',
				'ordering': ('-library', 'rank'),
			},
		),
		migrations.CreateModel(
			name='Library',
			fields=[
				('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
				('created', models.DateTimeField(db_index=True, editable=False, verbose_name='Created')),
				('updated', models.DateTimeField(db_index=True, editable=False, verbose_name='Updated')),
				('title', models.CharField(blank=True, max_length=255, verbose_name='Name')),
				('primary_attachment', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='attachments_library', to='django_attachments.Attachment', verbose_name='Primary attachment')),
			],
			options={
				'verbose_name_plural': 'Libraries',
				'verbose_name': 'Library',
			},
		),
		migrations.AddField(
			model_name='attachment',
			name='library',
			field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_attachments.Library', verbose_name='Library'),
		),
		migrations.AlterIndexTogether(
			name='attachment',
			index_together=set([('library', 'rank')]),
		),
	]



# -*- coding: utf-8 -*-
from django import forms
from django.db import models


class LibraryFormField(forms.ModelChoiceField):
	pass


class GalleryFormField(LibraryFormField):
	pass


class LibraryField(models.ForeignKey):
	description = "Attachments"

	def __init__(self, to='django_attachments.Library', *args, **kwargs):
		super().__init__(to, *args, **kwargs)

	def formfield(self, **kwargs):
		defaults = {'form_class': LibraryFormField}
		defaults.update(kwargs)
		return super().formfield(**defaults)


class GalleryField(LibraryField):
	description = "Gallery"

	def formfield(self, **kwargs):
		defaults = {'form_class': GalleryFormField}
		defaults.update(kwargs)
		return super().formfield(**defaults)



# -*- coding: utf-8 -*-
import os
import warnings

warnings.resetwarnings()
warnings.filterwarnings('ignore', message='Not importing directory .*', module='django.utils.encoding')
showwarning = warnings.showwarning


BASE_DIR = os.path.abspath(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))


def warn_with_filter(message, category, filename, *args):
	#if not os.path.abspath(filename).startswith(BASE_DIR):
	#	return
	showwarning(message, category, filename, *args)

warnings.showwarning = warn_with_filter

import logging

from django.contrib.staticfiles.handlers import StaticFilesHandler
from django.core.wsgi import get_wsgi_application
from django.template import TemplateSyntaxError
from django.views import debug
from django.views.debug import ExceptionReporter
from django.http.response import HttpResponse
from django_extensions.management.utils import RedirectHandler
from werkzeug.debug import DebuggedApplication


os.environ.setdefault("DJANGO_SETTINGS_MODULE", "web.settings")

logging.getLogger(__name__)

werklogger = logging.getLogger('werkzeug')
werklogger.setLevel(logging.INFO)
werklogger.addHandler(RedirectHandler(__name__))
werklogger.propagate = False


def forward_technical_500_response(request, exc_type, exc_value, tb, **kwargs):
	if request.META['REMOTE_ADDR'] == '127.0.0.1' and exc_type != TemplateSyntaxError:
		raise #pylint: disable=misplaced-bare-raise
	else:
		reporter = ExceptionReporter(request, exc_type, exc_value, tb)
		html = reporter.get_traceback_html()
		return HttpResponse(html, status=500, content_type='text/html')


debug.technical_500_response = forward_technical_500_response


application = DebuggedApplication(StaticFilesHandler(get_wsgi_application()), True)



# Generated by Django 4.2.1 on 2023-05-10 15:51

from django.db import migrations


class Migration(migrations.Migration):

	dependencies = [
		('django_attachments', '0003_attachment_options'),
	]

	operations = [
		migrations.RenameIndex(
			model_name='attachment',
			new_name='django_atta_library_9d0164_idx',
			old_fields=('library', 'rank'),
		),
	]



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.contrib.auth.mixins import UserPassesTestMixin
from django.http.response import JsonResponse
from django.shortcuts import get_object_or_404
from django.views.generic import CreateView, TemplateView
from django.urls import path

from .fields import LibraryField, GalleryField
from .forms import ImageUploadForm
from .models import Attachment, Library
from .views import AttachmentEditableMixin
from .widgets import AdminLibraryWidget, AdminGalleryWidget


class AttachmentsPermsMixin(UserPassesTestMixin):
	def test_func(self):
		user = self.request.user
		if not user.is_authenticated:
			return False
		if not user.is_staff:
			return False
		if not user.has_perm('attachments.change_attachment'):
			return False
		return True


class LibraryEditView(AttachmentsPermsMixin, AttachmentEditableMixin, TemplateView):
	template_name = 'admin/attachments/library_edit.html'

	def get_library(self):
		return get_object_or_404(Library, pk=self.kwargs['pk'])


class GalleryEditView(LibraryEditView):
	upload_form_class = ImageUploadForm


class AttachmentsAdminMixin(object):
	def formfield_for_dbfield(self, db_field, request, **kwargs):
		if isinstance(db_field, GalleryField):
			return db_field.formfield(widget=AdminGalleryWidget, **kwargs)
		if isinstance(db_field, LibraryField):
			return db_field.formfield(widget=AdminLibraryWidget, **kwargs)
		return super().formfield_for_dbfield(db_field, request, **kwargs)


class LibraryCreateViw(AttachmentsPermsMixin, CreateView):
	template_name = 'admin/attachments/library_create.html'
	model = Library
	fields = ('title',)

	def form_valid(self, form):
		self.object = form.save()
		return JsonResponse({'id': self.object.pk})


class AttachmentsInline(admin.TabularInline):
	model = Attachment
	fields = ('rank',)


class LibraryAdmin(admin.ModelAdmin):
	inlines = [AttachmentsInline]
	raw_id_fields = ('primary_attachment',)

	def get_urls(self):
		urlpatterns = super().get_urls()
		urlpatterns = [
			path('api/create/', LibraryCreateViw.as_view(), name='attachments_library_create_api'),
			path('api/attachments/<int:pk>/', LibraryEditView.as_view(), name='attachments_library_edit_api'),
			path('api/gallery/<int:pk>/', GalleryEditView.as_view(), name='attachments_gallery_edit_api'),
		] + urlpatterns
		return urlpatterns


admin.site.register(Attachment)
admin.site.register(Library, LibraryAdmin)



# -*- coding: utf-8 -*-
from django import forms
from django.forms.models import modelformset_factory
from django.utils.translation import gettext_lazy as _

from .models import Attachment


class AttachmentUploadForm(forms.ModelForm):
	def __init__(self, *args, **kwargs):
		self.library = kwargs.pop('library')
		super().__init__(*args, **kwargs)

	class Meta:
		model = Attachment
		fields = ('file',)

	def save(self, commit=True):
		obj = super().save(commit=False)
		obj.library = self.library
		if commit:
			obj.save()
		return obj


class ImageUploadForm(AttachmentUploadForm):
	file = forms.ImageField(
		label=_("Image")
	)

	class Meta(AttachmentUploadForm.Meta):
		pass


class AttachmentUpdateForm(forms.ModelForm):
	class Meta:
		model = Attachment
		fields = ()

	def save(self, commit=True):
		obj = super().save(commit=False)
		old_rank = obj.rank
		obj.rank = self.cleaned_data['ORDER'] - 1
		if commit:
			if old_rank != obj.rank:
				Attachment.objects.filter(pk=obj.pk).update(rank=obj.rank)
		return obj


AttachmentUpdateFormSet = modelformset_factory(
	Attachment,
	AttachmentUpdateForm,
	can_order=True,
	can_delete=True,
	extra=0
)



# -*- coding: utf-8 -*-
import mimetypes
import os
from io import BytesIO

from PIL import Image
from django.contrib.staticfiles import finders
from django.core.files.uploadedfile import SimpleUploadedFile
from django.templatetags.static import static


def check_ajax(request):
	if not hasattr(request, '_django_attachments_ajax'):
		accept = request.META.get('HTTP_ACCEPT', '')
		accept_types = set(
			mime.split(';')[0].strip()
			for mime in accept.split(',')
			if mime.split(';')[0].strip()
		)
		request._django_attachments_ajax = 'application/json' in accept_types
	return request._django_attachments_ajax


def resized_picture_field(self, image_field, max_image_size=None, force_image_type=None, strip_metadata=False, image_quality=70): # pylint: disable=unused-argument
	if max_image_size is None and force_image_type is None:
		return image_field
	image_file = BytesIO(image_field.read())
	image = Image.open(image_file).convert('RGB')

	if strip_metadata:
		image_without_exif = Image.new(image.mode, image.size)
		image_without_exif.putdata(image.getdata())
		image_without_exif.format = image.format
		image = image_without_exif

	if max_image_size is not None:
		if image.size[0] > max_image_size[0] or image.size[1] > max_image_size[1]:
			image.thumbnail(max_image_size, Image.ANTIALIAS)

	image_file = BytesIO()
	image.save(image_file, format=force_image_type or image.format, quality=image_quality)
	image_field.file = SimpleUploadedFile(image_field.name, image_file.getvalue())

	return image_field


def parse_mimetype(filename):
	mimetype = (mimetypes.guess_type(filename)[0] or '')[:200]
	mime_components = [d for d in mimetype.split('/') if d != '..' and d != '']
	if mime_components:
		mime_url_part = '/'.join(mime_components)
		safe_mimetype = os.path.join(*mime_components)
		mime_url = 'django_attachments/img/mimetypes/%s.png' % mime_url_part
		result = finders.find(os.path.join('django_attachments', 'img', 'mimetypes', safe_mimetype) + '.png')
	else:
		result = None
	if result is None:
		mime_url = 'django_attachments/img/mimetypes/application/octet-stream.png'
	return {
		'mimetype': mimetype,
		'mimetype_url': static(mime_url)
	}



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-01 13:16
from django.db import migrations, models
import django.db.models.deletion
import django_attachments.fields


class Migration(migrations.Migration):

	initial = True

	dependencies = [
		('django_attachments', '0001_initial'),
	]

	operations = [
		migrations.CreateModel(
			name='Article',
			fields=[
				('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
				('title', models.CharField(max_length=100)),
				('attachments', django_attachments.fields.LibraryField(on_delete=django.db.models.deletion.CASCADE, related_name='articles_with_attachment', to='django_attachments.Library')),
				('gallery', django_attachments.fields.GalleryField(on_delete=django.db.models.deletion.CASCADE, related_name='articles_with_gallery', to='django_attachments.Library')),
			],
			options={
				'verbose_name': 'Article',
				'verbose_name_plural': 'Articles',
			},
		),
	]



# -*- coding: utf-8 -*-
from django.db import models
from django_attachments.fields import LibraryField, GalleryField


class Article(models.Model):
	title = models.CharField(max_length=100)
	attachments = LibraryField(related_name='articles_with_attachment', on_delete=models.CASCADE)
	gallery = GalleryField(related_name='articles_with_gallery', on_delete=models.CASCADE)

	class Meta:
		verbose_name = "Article"
		verbose_name_plural = "Articles"

	def __str__(self):
		return self.title



# -*- coding: utf-8 -*-
from django.contrib import admin

from .admin_forms import ArticleForm
from .models import Article
from django_attachments.admin import AttachmentsAdminMixin


class ArticleAdmin(AttachmentsAdminMixin, admin.ModelAdmin):
	form = ArticleForm


admin.site.register(Article, ArticleAdmin)



# -*- coding: utf-8 -*-
from django.http.response import HttpResponse
from django.utils.functional import cached_property
from django.views.generic import TemplateView
from django_attachments.forms import ImageUploadForm
from django_attachments.models import Library
from django_attachments.views import AttachmentEditableMixin

from .models import Article


class IndexView(TemplateView):
	template_name = 'index.html'


class GetOrCreateArticleMixin(object):
	article_title = 'Test'

	@cached_property
	def article(self):
		article = Article.objects.filter(title=self.article_title).first()
		if article is None:
			attachments = Library()
			attachments.save()
			gallery = Library()
			gallery.save()
			article = Article(title=self.article_title, attachments=attachments, gallery=gallery)
			article.save()
		return article


class LiveUploadAttachments(AttachmentEditableMixin, GetOrCreateArticleMixin, TemplateView):
	template_name = 'live_upload_attachments.html'

	def get_library(self):
		return self.article.attachments


class OnSaveUploadAttachments(AttachmentEditableMixin, GetOrCreateArticleMixin, TemplateView):
	article_title = 'Test 2'
	template_name = 'on_save_upload_attachments.html'

	def get_library(self):
		return self.article.attachments

	def post(self, request):
		action = self.request.POST.get('action')
		if action not in ('upload', 'update', 'mimetype'):
			return HttpResponse('')
		return super().post(request)



class GalleryUpload(AttachmentEditableMixin, GetOrCreateArticleMixin, TemplateView):
	template_name = 'gallery_upload.html'
	upload_form_class = ImageUploadForm

	def get_library(self):
		return self.article.gallery



# Generated by Django 3.0.3 on 2020-05-22 13:45

from django.db import migrations, models


class Migration(migrations.Migration):

	dependencies = [
		('django_attachments', '0002_auto_20180727_0804'),
	]

	operations = [
		migrations.AddField(
			model_name='attachment',
			name='options',
			field=models.TextField(blank=True, verbose_name='Options'),
		),
	]



# -*- coding: utf-8 -*-
from django import forms
from django.urls import reverse


class AdminLibraryWidget(forms.Widget):
	template_name = 'django_attachments/widgets/admin_attachments.html'

	class Media:
		css = {
			'all': ('django_attachments/css/attachments.css',)
		}
		js = (
			'django_attachments/vendor/dropzone/dropzone.min.js',
			'django_attachments/vendor/sortable/Sortable.min.js',
			'django_attachments/js/attachments.js',
			'django_attachments/js/admin_attachments.js',
		)

	def get_context(self, name, value, attrs):
		context = super().get_context(name, value, attrs)
		create_url = reverse('admin:attachments_library_create_api')
		library_url = reverse('admin:attachments_library_edit_api', args=('0000',))
		# hack
		library_url = library_url.replace('0000', '__library_id__')
		context['create_url'] = create_url
		context['library_url'] = library_url
		return context


class AdminGalleryWidget(AdminLibraryWidget):
	def get_context(self, name, value, attrs):
		context = super().get_context(name, value, attrs)
		library_url = reverse('admin:attachments_gallery_edit_api', args=('0000',))
		# hack
		library_url = library_url.replace('0000', '__library_id__')
		context['library_url'] = library_url
		return context



# -*- coding: utf-8 -*-
from django import apps
from django.utils.translation import gettext_lazy as _


class AppConfig(apps.AppConfig):
	name = 'django_attachments'
	verbose_name = _("Attachemnts")
	default_auto_field = 'django.db.models.AutoField'

	def ready(self):
		from .cleanup import register_cleaner_for_model
		register_cleaner_for_model(self.get_model('Attachment'))



# -*- coding: utf-8 -*-
import mimetypes
from io import BytesIO
from os import path
from uuid import uuid4

from PIL import Image
from django.db import models
from django.db.models import F, Max, Subquery, OuterRef
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from easy_thumbnails.fields import ThumbnailerField

from .utils import parse_mimetype


class TimestampModelMixin(models.Model):
	created = models.DateTimeField(_("Created"), editable=False, db_index=True)
	updated = models.DateTimeField(_("Updated"), editable=False, db_index=True)

	def save(self, *args, **kwargs):
		self.updated = timezone.now()
		if not self.id and not self.created:
			self.created = self.updated
		return super().save(*args, **kwargs)

	class Meta:
		abstract = True


def upload_path_handler(instance, filename):
	pk = instance.library.pk
	filename = str(uuid4()) + path.splitext(filename)[1]
	return path.join('attachments', "{0:02x}".format(pk % 256), str(pk), filename)


class LibraryQuerySet(models.QuerySet):
	def update_primary_image(self):
		# Subquery to get the primary attachment for each library by lowest rank
		primary_attachment = Subquery(Attachment.objects
			.filter(library=OuterRef('pk'))
			.order_by('rank')
			.values('pk')[:1])
		return self.update(primary_attachment=primary_attachment)


class Library(TimestampModelMixin, models.Model):
	objects = LibraryQuerySet.as_manager()

	title = models.CharField(
		verbose_name=_("Name"),
		max_length=255,
		blank=True
	)
	primary_attachment = models.ForeignKey(
		'Attachment',
		verbose_name=_("Primary attachment"),
		blank=True,
		null=True,
		on_delete=models.SET_NULL,
		related_name='attachments_library'
	)

	class Meta:
		verbose_name = _("Library")
		verbose_name_plural = _("Libraries")

	def __str__(self):
		if self.title:
			return self.title
		elif self.pk:
			return 'Library #%d' % self.pk
		else:
			return 'Library'


class AttachmentQuerySet(models.QuerySet):
	def images(self):
		return self.filter(image_width__isnull=False)


class Attachment(TimestampModelMixin, models.Model):
	objects = AttachmentQuerySet.as_manager()

	library = models.ForeignKey(
		Library,
		verbose_name=_("Library"),
		on_delete=models.CASCADE
	)
	rank = models.IntegerField(
		verbose_name=_("Rank")
	)
	original_name = models.CharField(
		max_length=255,
		verbose_name=_("Original name")
	)
	file = ThumbnailerField(
		verbose_name=_("File"),
		upload_to=upload_path_handler
	)
	filesize = models.BigIntegerField(
		verbose_name=_("File size"),
	)
	mimetype = models.CharField(
		verbose_name=_("File type"),
		max_length=200,
		blank=True
	)
	image_width = models.IntegerField(
		verbose_name=_("Image width"),
		blank=True,
		null=True
	)
	image_height = models.IntegerField(
		verbose_name=_("Image height"),
		blank=True,
		null=True
	)
	title = models.CharField(
		verbose_name=_("Name"),
		max_length=255,
		blank=True
	)
	caption = models.TextField(
		verbose_name=_("Caption"),
		blank=True
	)
	options = models.TextField(
		verbose_name=_("Options"),
		blank=True
	)

	class Meta:
		verbose_name = _("Attachment")
		verbose_name_plural = _("Attachments")
		ordering = ('-library', 'rank')
		indexes = [
			models.Index(fields=['library', 'rank'])
		]

	def __str__(self):
		return self.original_name

	@property
	def is_image(self):
		return self.image_width is not None

	@property
	def mimetype_info(self):
		return parse_mimetype(self.original_name)

	def save(self, *args, **kwargs):
		if not self.original_name:
			self.original_name = self.file.name
		if self.original_name:
			self.original_name = self.original_name[:255]
			self.mimetype = (mimetypes.guess_type(self.original_name)[0] or '')[:200]
		if self.rank is None:
			next_rank = self._rank_queryset().aggregate(max_rank=Max('rank'))['max_rank']
			if next_rank is None:
				next_rank = 0
			else:
				next_rank += 1
			self.rank = next_rank
		else:
			if self.pk is None:
				self._rank_queryset().filter(rank__gte=self.rank).update(rank=F('rank')+1)
		try:
			self.filesize = self.file.size
		except Exception:
			self.filesize = -1
		if self.file and self.filesize >= 0:
			source = BytesIO(self.file.read())
			try:
				image = Image.open(source)
				self.image_width, self.image_height = image.size
			except IOError:
				pass
			finally:
				self.file.seek(0)
		else:
			self.filesize = -1
			self.image_width = None
			self.image_height = None
		return super().save(*args, **kwargs)

	def delete(self, *args, **kwargs):
		self._rank_queryset().filter(rank__gt=self.rank).update(rank=F('rank')-1)
		return super().delete(*args, **kwargs)

	def move_to(self, position):
		if position == self.rank:
			return
		if self.rank > position:
			self._rank_queryset().filter(rank__lt=self.rank, rank__gte=position).update(rank=F('rank')+1)
		else:
			self._rank_queryset().filter(rank__gt=self.rank, rank__lte=position).update(rank=F('rank')-1)
		self.rank = position
		self.save()

	def _rank_queryset(self):
		return Attachment.objects.filter(library=self.library).order_by('rank')



# Generated by Django 2.0.6 on 2018-07-27 08:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_attachments', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='attachment',
            name='caption',
            field=models.TextField(blank=True, verbose_name='Caption'),
        ),
        migrations.AddField(
            model_name='attachment',
            name='title',
            field=models.CharField(blank=True, max_length=255, verbose_name='Name'),
        ),
    ]



# -*- coding: utf-8 -*-
import logging

from django.core.files.storage import get_storage_class
from django.db import models
from django.db.models.signals import post_delete, pre_save
from easy_thumbnails.conf import settings
from easy_thumbnails.files import get_thumbnailer


logger = logging.getLogger('django.db.models')


def delete_files(sender, instance, *args, **kwargs): #pylint: disable=unused-argument
	for field in instance._meta.fields:
		if not isinstance(field, models.FileField):
			continue
		file_to_delete = getattr(instance, field.name)
		delete_file(file_to_delete)


def delete_old_files(sender, instance, *args, **kwargs): #pylint: disable=unused-argument
	if not instance.pk:
		return

	try:
		old_instance = instance.__class__.objects.get(pk=instance.pk)
	except instance.__class__.DoesNotExist:
		return

	for field in instance._meta.fields:
		if not isinstance(field, models.FileField):
			continue
		old_file = getattr(old_instance, field.name)
		new_file = getattr(instance, field.name)
		if new_file != old_file:
			delete_file(old_file)


def delete_file(file_instance):
	if not file_instance.name:
		return
	thumbnailer = get_thumbnailer(file_instance)
	thumbnailer.thumbnail_storage = get_storage_class(settings.THUMBNAIL_DEFAULT_STORAGE)()
	thumbnailer.delete_thumbnails()
	storage = file_instance.storage
	if storage and storage.exists(file_instance.name):
		try:
			storage.delete(file_instance.name)
		except IOError:
			logger.error('File not deleted: %s', file_instance.name)


def register_cleaner_for_model(model_cls):
	post_delete.connect(delete_files, sender=model_cls)
	pre_save.connect(delete_old_files, sender=model_cls)



# -*- coding: utf-8 -*-
import glob
import pathlib
import subprocess
import sys

from setuptools import setup


def create_mo_files():
	prefix = pathlib.Path('wtextra') / 'delivery_zasielkovna'

	for po_file in glob.glob(str(pathlib.Path(prefix) / 'locale' / '*' / 'LC_MESSAGES' / 'django.po')):
		sys.stdout.write(f"Compiling {po_file}")
		compiled = subprocess.check_output(['msgfmt', '--check-format', po_file, '-o', '-'])
		sys.stdout.write(" .. OK\n")
		with open(po_file[:-3] + '.mo', 'wb') as fp:
			fp.write(compiled)


setup(
	data_files=create_mo_files(),
	use_scm_version=True,
)



# -*- coding: utf-8 -*-
from django.conf import settings
from django.conf.urls.static import static
from django.contrib import admin
from django.urls import path

from example import views as example_views


urlpatterns = [
	path('admin/', admin.site.urls),
	path('', example_views.IndexView.as_view(), name='home'),
	path('attachments/', example_views.LiveUploadAttachments.as_view(), name='live_upload_attachments'),
	path('attachments-save/', example_views.OnSaveUploadAttachments.as_view(), name='on_save_upload_attachments'),
	path('gallery/', example_views.GalleryUpload.as_view(), name='gallery_upload'),
]

if settings.DEBUG:
	urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



# -*- coding: utf-8 -*-
from django import forms
from django_attachments.models import Library

from .models import Article


class ArticleForm(forms.ModelForm):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.fields['attachments'].required = False
		self.fields['gallery'].required = False

	def save(self, commit=True):
		obj = super().save(commit=False)
		if not hasattr(obj, 'attachments'):
			lib = Library()
			lib.save()
			obj.attachments = lib
		if not hasattr(obj, 'gallery'):
			lib = Library()
			lib.save()
			obj.gallery = lib
		if commit:
			obj.save()
		return obj

	class Meta:
		model = Article
		fields = '__all__'



# -*- coding: utf-8 -*-
import json

from django.core import signing
from django.http import JsonResponse
from django.http.response import HttpResponseRedirect
from django.utils.functional import cached_property
from easy_thumbnails.exceptions import EasyThumbnailsError
from easy_thumbnails.files import get_thumbnailer

from .forms import AttachmentUploadForm, AttachmentUpdateFormSet
from .models import Attachment
from .utils import parse_mimetype, check_ajax


class AttachmentEditableMixin(object):
	upload_form_class = AttachmentUploadForm
	update_form_class = AttachmentUpdateFormSet
	thumbnail_options = {
		'thumbnail': {'crop': True, 'size': (200, 150)},
	}

	def can_upload_attachment(self):
		return True

	def can_update_attachment(self):
		return True

	def get_library(self):
		raise NotImplementedError

	def get_gallery_field_name(self):
		return 'gallery'

	def get_attachment_form_kwargs(self, action, **extra_kwargs):
		kwargs = {}
		kwargs.update(extra_kwargs)
		if self.request.POST.get('action') == action:
			kwargs.update({
				'data': self.request.POST,
				'files': self.request.FILES,
			})
		return kwargs

	def get_upload_form_kwargs(self):
		return self.get_attachment_form_kwargs('upload', library=self.get_library())

	def get_update_form_kwargs(self):
		library = self.get_library()
		if library and library.pk:
			queryset = library.attachment_set.all()
		else:
			queryset = Attachment.objects.none()
		return self.get_attachment_form_kwargs('update', queryset=queryset)

	@cached_property
	def upload_form(self):
		if self.can_upload_attachment():
			return self.upload_form_class(**self.get_upload_form_kwargs())
		else:
			return None

	@cached_property
	def update_form(self):
		if self.can_update_attachment():
			return self.update_form_class(**self.get_update_form_kwargs())
		else:
			return None

	def get_context_data(self, **kwargs):
		ctx = super().get_context_data(**kwargs)
		ctx['upload_form'] = self.upload_form
		ctx['update_form'] = self.update_form
		return ctx

	def post(self, request, *args, **kwargs):
		action = self.request.POST.get('action')
		if action == 'upload' and self.upload_form:
			if self.upload_form.is_valid():
				return self.upload_form_valid(self.upload_form)
			else:
				return self.upload_form_invalid(self.upload_form)
		if action == 'update' and self.upload_form:
			if self.update_form.is_valid():
				return self.update_form_valid(self.update_form)
			else:
				return self.update_form_invalid(self.update_form)
		if action == 'mimetype':
			filename = self.request.POST.get('filename')
			mimetype = parse_mimetype(filename)
			return JsonResponse(mimetype)
		if action == 'get_library':
			lib = self.get_library()
			if lib.pk is None:
				lib.save()
			field_name = self.get_gallery_field_name()
			signer = signing.Signer(salt='attachment_' + field_name)
			return JsonResponse({'id': lib.pk, 'name': field_name, 'sign': signer.sign(str(lib.pk))})
		return super().post(request, *args, **kwargs)

	def upload_form_valid(self, form):
		upload = form.save()
		self.update_primary_attachment()
		if check_ajax(self.request):
			attachments = self.serialize_attachemnts()
			for attachment in attachments:
				attachment['is_new'] = upload.id == attachment['id']
			return JsonResponse({'attachments': attachments})
		return HttpResponseRedirect(self.request.get_full_path())

	def upload_form_invalid(self, form):
		if check_ajax(self.request):
			return JsonResponse({'errors': json.loads(form.errors.as_json())})
		return self.render_to_response(self.get_context_data())

	def update_form_valid(self, form):
		form.save()
		self.update_primary_attachment()
		if check_ajax(self.request):
			return self.render_json_attachments()
		return HttpResponseRedirect(self.request.get_full_path())

	def update_form_invalid(self, form):
		if check_ajax(self.request):
			errors = {}
			for subform in form:
				errors.update(json.loads(subform.errors.as_json()))
			return JsonResponse({'errors': errors})
		return self.render_to_response(self.get_context_data())

	def get(self, request, *args, **kwargs):
		if check_ajax(self.request):
			return self.render_json_attachments()
		return super().get(request, *args, **kwargs)

	def serialize_attachemnts(self):
		library = self.get_library()
		if not library or not library.pk:
			return []
		attachments = library.attachment_set.all()
		attachments_data = []
		for attachment in attachments:
			attachment_data = {
				'id': attachment.pk,
				'name': attachment.original_name,
				'rank': attachment.rank,
				'filesize': attachment.filesize,
				'url': attachment.file.url,
				'mimetype': attachment.mimetype,
				'mimetype_url': parse_mimetype(attachment.original_name)['mimetype_url'],
			}
			if attachment.is_image:
				attachment_data['image_width'] = attachment.image_width
				attachment_data['image_height'] = attachment.image_height
				thumbnailer = get_thumbnailer(attachment.file)
				for key, options in self.thumbnail_options.items():
					try:
						attachment_data[key] = thumbnailer.get_thumbnail(options).url
					except EasyThumbnailsError:
						attachment_data[key] = None
			attachments_data.append(attachment_data)
		return attachments_data

	def render_json_attachments(self):
		return JsonResponse({'attachments': self.serialize_attachemnts()})

	def update_primary_attachment(self):
		library = self.get_library()
		if not library.pk:
			return
		library.refresh_from_db()
		first_attachment = library.attachment_set.order_by('rank').first()
		if library.primary_attachment != first_attachment:
			library.primary_attachment = first_attachment
			library.save()



# -*- coding: utf-8 -*-
from io import BytesIO
import os

from easy_thumbnails.files import get_thumbnailer
from PIL import Image
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase

from .models import Library, Attachment


class AttachmentModelTest(TestCase):
	def setUp(self):
		pass

	def create_library(self, title='Test'):
		library = Library(title=title)
		library.save()
		return library

	def create_attachment(self, filename, data, library=None, **kwargs):
		if library == None:
			library = self.create_library()
		uploaded_file = SimpleUploadedFile(filename, data)
		attachment = Attachment(file=uploaded_file, library=library, **kwargs)
		attachment.save()
		return attachment

	def create_image(self, image_size):
		im = Image.new('RGB', image_size)
		data = BytesIO()
		im.save(data, 'JPEG')
		return data.getvalue()

	def test_library(self):
		title = 'Test'
		library = self.create_library()
		self.assertEqual(library.title, title)
		self.assertIsNone(library.primary_attachment)
		library.delete()

	def test_upload_file(self):
		data = b'data'
		attachment = self.create_attachment('upload.txt', data)
		self.assertEquals(attachment.original_name, 'upload.txt')
		self.assertEquals(attachment.rank, 0)
		self.assertEquals(attachment.filesize, len(data))
		self.assertIsNone(attachment.image_width)
		attachment.delete()

	def test_upload_image(self):
		image_size = (5, 10)
		attachment = self.create_attachment('image.jpg', self.create_image(image_size))
		self.assertEquals(attachment.image_width, image_size[0])
		self.assertEquals(attachment.image_height, image_size[1])
		attachment.delete()

	def test_rank_create_delete(self):
		library = self.create_library()
		attachments = [
			self.create_attachment('upload.txt', b'', library),
			self.create_attachment('upload.txt', b'', library),
		]
		self.assertEquals(attachments[0].rank, 0)
		self.assertEquals(attachments[1].rank, 1)

		# ranks from other library
		library2 = self.create_library()
		attachments2 = [
			self.create_attachment('upload.txt', b'', library2),
			self.create_attachment('upload.txt', b'', library2),
		]
		self.assertEquals(attachments2[0].rank, 0)
		self.assertEquals(attachments2[1].rank, 1)

		# change rank when delete
		attachments[0].delete()
		attachments[1].refresh_from_db()
		self.assertEquals(attachments[1].rank, 0)
		attachments.pop(0)
		# preserve rank when delete last
		attachments2[1].delete()
		attachments2[0].refresh_from_db()
		self.assertEquals(attachments2[0].rank, 0)
		attachments2.pop(1)

		# insert at specific position
		attachment = self.create_attachment('upload.txt', b'', library, rank=0)
		attachments[0].refresh_from_db()
		attachments.insert(0, attachment)
		self.assertEquals(attachments[0].rank, 0)
		self.assertEquals(attachments[1].rank, 1)

		attachments[0].delete()
		attachments[1].delete()
		attachments2[0].delete()

	def test_rank_move(self):
		library = self.create_library()
		count = 5
		attachments = []
		result_order = list(range(5))
		for i in range(count):
			attachments.append(self.create_attachment('upload.txt', b'', library))

		for i in range(count):
			attachments[i].refresh_from_db()
			self.assertEquals(attachments[i].rank, result_order[i])

		attachments[2].move_to(2) # position not changed

		for i in range(count):
			attachments[i].refresh_from_db()
			self.assertEquals(attachments[i].rank, result_order[i])

		attachments[3].move_to(1)
		result_order = [0, 2, 3, 1, 4]

		for i in range(count):
			attachments[i].refresh_from_db()
			self.assertEquals(attachments[i].rank, result_order[i])
			# reset
			attachments[i].rank = i
			attachments[i].save()

		attachments[1].move_to(3)
		result_order = [0, 3, 1, 2, 4]

		for i in range(count):
			attachments[i].refresh_from_db()
			self.assertEquals(attachments[i].rank, result_order[i])
			# reset
			attachments[i].rank = i
			attachments[i].save()

		for i in range(count):
			attachments[i].delete()

	def test_delete(self):
		attachment = self.create_attachment('upload.png', self.create_image((5, 5)))
		thumbnailer = get_thumbnailer(attachment.file)
		thumbnail = thumbnailer.get_thumbnail({'crop': True, 'size': (1, 1)})

		self.assertTrue(os.path.exists(attachment.file.path))
		self.assertTrue(os.path.exists(thumbnail.path))

		attachment.delete()
		self.assertFalse(os.path.exists(attachment.file.path))
		self.assertFalse(os.path.exists(thumbnail.path))

	def test_update_library(self):
		library = self.create_library()
		attachment = self.create_attachment('upload.txt', '', library=library)
		library.refresh_from_db()
		self.assertIsNone(library.primary_attachment)
		Library.objects.filter(pk=library.pk).update_primary_image()
		library.refresh_from_db()
		self.assertEqual(library.primary_attachment, attachment)
